﻿using System;
using System.Windows.Forms;

namespace DesktopTools.BaseControls
{
	/// <summary>
	/// Контрол, предоставляющий работу с bool-полем
	/// </summary>
	public partial class BaseControlBool : AbstractBaseControl
	{
		/// <summary>
		/// Конструктор
		/// </summary>
		/// <param name="propertyName"></param>
		/// <param name="mustFilling"></param>
		/// <param name="readOnly"></param>
		public BaseControlBool(string propertyName, bool mustFilling, bool readOnly) : base(propertyName, mustFilling, readOnly)
		{
			InitializeComponent();

			if (!_mustFilling)
			{
				checkBoxNullable.CheckedChanged += (object sender, EventArgs e) =>
				{
					checkBox.Enabled = !(sender as CheckBox).Checked;
					CallOnValueChangeEvent();
				};
				panelControl.Controls.Add(checkBoxNullable);
			}

			checkBox.CheckedChanged += (object sender, EventArgs e) => { CallOnValueChangeEvent(); };
			panelControl.Controls.Add(checkBox);
		}

		protected override void SetDefaultValue() => _originalValue = _mustFilling ? false : null;

		protected override void SetValueToControl(object value)
		{
			if (value != null)
			{
				checkBox.Checked = Convert.ToBoolean(value);
			}
			else if (!_mustFilling)
			{
				checkBoxNullable.Checked = true;
			}
		}

		public override void DropValue()
		{
			if (_originalValue != null)
			{
				checkBox.Checked = Convert.ToBoolean(_originalValue);
			}
			else if (!_mustFilling)
			{
				checkBoxNullable.Checked = true;
			}
		}

		public override bool CheckValue() => true;

		protected override object GetValueFromControl() => !_mustFilling && checkBoxNullable.Checked ? null : checkBox.Checked;
	}
}