﻿using ModuleTools.Extensions;
using System;
using System.Windows.Forms;

namespace DesktopTools.BaseControls
{
	/// <summary>
	/// Контрол, предоставляющий работу с датой
	/// </summary>
	public partial class BaseControlDateTime : AbstractBaseControl
	{
		/// <summary>
		/// Конструктор
		/// </summary>
		/// <param name="propertyName"></param>
		/// <param name="mustFilling"></param>
		/// <param name="readOnly"></param>
		/// <param name="minDate"></param>
		/// <param name="maxDate"></param>
		/// <param name="customDateFormat"></param>
		public BaseControlDateTime(string propertyName, bool mustFilling, bool readOnly, DateTime? minDate, DateTime? maxDate, string customDateFormat) : base(propertyName, mustFilling, readOnly)
		{
			InitializeComponent();

			if (!_mustFilling)
			{
				checkBoxNullable.CheckedChanged += (object sender, EventArgs e) =>
				{
					dateTimePicker.Enabled = !(sender as CheckBox).Checked;
					CallOnValueChangeEvent();
				};
				panelControl.Controls.Add(checkBoxNullable);
			}

			dateTimePicker.ValueChanged += (object sender, EventArgs e) => { CallOnValueChangeEvent(); };
			if (minDate.HasValue)
			{
				dateTimePicker.MinDate = minDate.Value;
			}
			if (maxDate.HasValue)
			{
				dateTimePicker.MaxDate = maxDate.Value;
			}
			if(customDateFormat.IsNotEmpty())
			{
				dateTimePicker.CustomFormat = customDateFormat;
			}
			panelControl.Controls.Add(dateTimePicker);
		}

		protected override void SetDefaultValue() => _originalValue = _mustFilling ? DateTime.Now : null;

		protected override void SetValueToControl(object value)
		{
			if (value != null)
			{
				dateTimePicker.Value = Convert.ToDateTime(value);
			}
			else if (!_mustFilling)
			{
				checkBoxNullable.Checked = true;
			}
		}

		public override void DropValue()
		{
			if (_originalValue != null)
			{
				dateTimePicker.Value = Convert.ToDateTime(_originalValue);
			}
			else if (!_mustFilling)
			{
				checkBoxNullable.Checked = true;
			}
		}

		public override bool CheckValue() => true;

		protected override object GetValueFromControl() => !_mustFilling && checkBoxNullable.Checked ? null : Convert.ToDateTime(dateTimePicker.Value);
	}
}