﻿using System;
using System.Windows.Forms;

namespace DesktopTools.BaseControls
{
	/// <summary>
	/// Контрол, предоставляющий работу с целочисленным полем
	/// </summary>
	public partial class BaseControlInt : AbstractBaseControl
	{
		/// <summary>
		/// Конструктор
		/// </summary>
		/// <param name="propertyName"></param>
		/// <param name="mustFilling"></param>
		/// <param name="readOnly"></param>
		/// <param name="minValue"></param>
		/// <param name="maxValue"></param>
		public BaseControlInt(string propertyName, bool mustFilling, bool readOnly, decimal? minValue, decimal? maxValue) : base(propertyName, mustFilling, readOnly)
		{
			InitializeComponent();

			if (!_mustFilling)
			{
				checkBoxNullable.CheckedChanged += (object sender, EventArgs e) =>
				{
					numericUpDown.Enabled = !(sender as CheckBox).Checked;
					CallOnValueChangeEvent();
				};
				panelControl.Controls.Add(checkBoxNullable);
			}

			numericUpDown.ValueChanged += (object sender, EventArgs e) => { CallOnValueChangeEvent(); };
			if (minValue.HasValue)
			{
				numericUpDown.Minimum = minValue.Value;
			}
			if (maxValue.HasValue)
			{
				numericUpDown.Maximum = maxValue.Value;
			}
			panelControl.Controls.Add(numericUpDown);
		}

		protected override void SetDefaultValue() => _originalValue = _mustFilling ? 0 : null;

		protected override void SetValueToControl(object value)
		{
			if (value != null)
			{
				numericUpDown.Value = Convert.ToInt32(value);
			}
			else if (!_mustFilling)
			{
				checkBoxNullable.Checked = true;
			}
		}

		public override void DropValue()
		{
			if (_originalValue != null)
			{
				numericUpDown.Value = Convert.ToInt32(_originalValue);
			}
			else if (!_mustFilling)
			{
				checkBoxNullable.Checked = true;
			}
		}

		public override bool CheckValue() => true;

		protected override object GetValueFromControl() => !_mustFilling && checkBoxNullable.Checked ? null : Convert.ToInt32(numericUpDown.Value);
	}
}